
#include <SD.h>

#include <SoftwareSerial.h>

/* ****** Settings ****** */
char log_filename[13] = "data.log"; // keep this FAT friendly
// how long should we wait when not moving before we stop logging
#define STOP 60000
/* ****** End of user settings ****** */

#define PIN_GPS_ENABLE 7
SoftwareSerial gpsSerial(5,6);

#define PIN_SD_SS 10
File log_file;
boolean sd_available = false;

boolean moving = true;
unsigned long last_move = 0;
#define ZERO_X 512
#define ZERO_Y 512
#define ZERO_Z 512
#define M_THRESH 60000
#define PIN_X 4
#define PIN_Y 3
#define PIN_Z 2

void setup() {
    Serial.begin(9600);

    //GPS setup
    gpsSerial.begin(4800);
    pinMode(PIN_GPS_ENABLE,OUTPUT);
    digitalWrite(PIN_GPS_ENABLE,HIGH);

    //SD setup
    pinMode(PIN_SD_SS,OUTPUT);
    pinMode(10,OUTPUT);
    start_sd();

    start_gps();
}

void loop() {
    if(moving) {
        if(sd_available) {
            log_file = SD.open(log_filename,FILE_WRITE);
            if(!log_file) {
                sd_available = false;
            }
        }
        if(gpsSerial.overflow()) {
            if(sd_available) {
                log_file.write("\r\n$PXXXX,SoftwareSerial overflow!!!");
            } else {
                Serial.println("\r\n$PXXXX,SoftwareSerial overflow!!!");
            }
        }
        while(gpsSerial.available()) {
            int r = gpsSerial.read();
            if(sd_available) {
                log_file.write(r);
            } else {
                Serial.write(r);
            }
        }
        check_movement();
        log_file.close();
    } else {
        delay(1000);
        check_movement();
    }
}

void start_gps() {
    gpsSerial.listen();
    digitalWrite(PIN_GPS_ENABLE,LOW);
}

void stop_gps() {
    digitalWrite(PIN_GPS_ENABLE,HIGH);
}

void start_sd() {
    if (!SD.begin(PIN_SD_SS)) {
        Serial.println("Card failed, or not present");
    } else {
        log_file = SD.open(log_filename,FILE_WRITE);
        if (log_file) {
            log_file.close();
            sd_available = true;
        } else {
            Serial.print("Can't open file <");
            Serial.print(log_filename);
            Serial.println("> for writing.");
        }
    }
}

void check_movement() {
    int x = analogRead(PIN_X) - ZERO_X;
    int y = analogRead(PIN_Y) - ZERO_Y;
    int z = analogRead(PIN_Z) - ZERO_Z;
    long acc = (long)x*x + (long)y*y + (long)z*z;
    if (acc > M_THRESH) {
        last_move = millis();
        if (! moving) {
            start_gps();
            start_sd();
            // Ignore data that has overflowed
            gpsSerial.overflow();
        }
        moving = true;
    } else {
        if (moving) {
            unsigned long now = millis();
            // TODO: check for the overflow condition of millis 
            // (or reboot the arduino every 49 days :) )
            if (now-last_move > STOP) {
                moving = false;
                stop_gps();
                if(sd_available) {
                    log_file.write("\r\n$PXXXX,Device is idle, stop logging\r\n");
                } else {
                    Serial.println("\r\n$PXXXX,Device is idle, stop logging\r\n");
                }
            }
        }
    }
}
